@@ -3,11 +3,8 @@
  * Horde_ActiveSync_Device::
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2013-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2010-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -15,11 +12,8 @@
  * Horde_ActiveSync_Device:: Wraps all functionality related to device data.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2010-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2010-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -79,6 +73,7 @@ class Horde_ActiveSync_Device
     const TYPE_TOUCHDOWN       = 'touchdown';
     const TYPE_UNKNOWN         = 'unknown';
     const TYPE_NINE            = 'nine';
+    const TYPE_GMAIL           = 'gmail';
 
     /**
      * Quirk to specify if the client fails to property ghost the
@@ -93,6 +88,13 @@ class Horde_ActiveSync_Device
      */
     const QUIRK_INCORRECTLY_SENDS_EMPTY_PICTURE_TAG = 2;
 
+    /**
+     * These client can support TNEF data without decoding it first.
+     *
+     * @since 2.40.0
+     */
+    const QUIRK_SUPPORTS_TNEF = 3;
+
     /**
      * Device properties.
      *
@@ -518,7 +520,11 @@ class Horde_ActiveSync_Device
     /**
      * Return the number of hours to offset a POOMCONTACTS:BIRTHDAY
      * or ANNIVERSARY field in an attempt to work around a bug in the
-     * protocol - which doesn't define a standard time for birthdays to occur.
+     * protocol - the protocol states that "the time portion SHOULD be ignored,
+     * so that synchronizing between different timezones does not change the
+     * date" (MS-ASCNTC 2.2.2.6). However, it appears that next to zero clients
+     * actually honor this.
+     *
      *
      *  WP:
      *     Devices seem to send the birthdays at the entered date, with
@@ -539,6 +545,15 @@ class Horde_ActiveSync_Device
      *     midnight on date_default_timezone() converted to UTC.
      *
      *   Android:
+     *     Lollipop and newer got rid of the stock "Email" app in favor of
+     *     GMail. Gmail seems to send birthdays up at 12:00:00 UTC, but doesn't
+     *     expect them at that time. It *seems* that sending them down at
+     *     00:00:00 UTC works, but Unfortunately I don't know the version
+     *     history here as to if they always did this, so will rely on bug
+     *     reports if this is not correct :)
+     *
+     *     For legacy android:
+     *
      *     For contacts originating on the SERVER, the following is true:
      *
      *     Stock 4.3 Takes the down-synched bday value which is assumed to be
@@ -590,10 +605,16 @@ class Horde_ActiveSync_Device
         case 'wp':  // Legacy. Remove in H6.
             if ($toEas) {
                 return new Horde_Date($date->format('Y-m-d'), 'UTC');
-            } else {
-                $date = new Horde_Date($date->format('Y-m-d'));
-                return $date->setTimezone('UTC');
             }
+            $date = new Horde_Date($date->format('Y-m-d'));
+            return $date->setTimezone('UTC');
+
+        case self::TYPE_GMAIL:
+            if ($toEas) {
+                return new Horde_Date($date->format('Y-m-d'), 'UTC');
+            }
+            $date = new Horde_Date($date->format('Y-m-d'));
+            return $date->setTimezone('UTC');
 
         case self::TYPE_ANDROID:
             // Need to protect against clients that don't send the actual Android
@@ -610,15 +631,14 @@ class Horde_ActiveSync_Device
             if ($this->getMajorVersion() >= 4 && $this->getMajorVersion() <= 10) {
                 if ($toEas) {
                     return new Horde_Date($date->format('Y-m-d 08:00:00'), 'UTC');
-                } else {
-                    $date = new Horde_Date($date->format('Y-m-d'));
-                    return $date->setTimezone('UTC');
                 }
-            } else {
-                // POOMCONTACTS:BIRTHDAY not really supported in early Android
-                // versions. Return as is.
-                return $date;
+                $date = new Horde_Date($date->format('Y-m-d'));
+                return $date->setTimezone('UTC');
+
             }
+            // POOMCONTACTS:BIRTHDAY not really supported in early Android
+            // versions. Return as is.
+            return $date;
 
         case self::TYPE_IPAD:
         case self::TYPE_IPHONE:
@@ -626,32 +646,29 @@ class Horde_ActiveSync_Device
             if ($this->getMajorVersion() >= 5) {
                 // iOS >= 5 handles it correctly more or less.
                 if ($toEas) {
-                    return new Horde_Date($date->format('Y-m-d 00:00:00'));
-                } else {
-                    return $date;
-                }
-            } else {
-                if ($toEas) {
-                    return new Horde_Date($date->format('Y-m-d'), 'UTC');
-                } else {
-                    return new Horde_Date($date->format('Y-m-d'));
+                    return new Horde_Date($date->format('Y-m-d 00:00:00'), 'UTC');
                 }
+                $date = new Horde_Date($date->format('Y-m-d'));
+                return $date->setTimezone('UTC');
             }
+            if ($toEas) {
+                return new Horde_Date($date->format('Y-m-d'), 'UTC');
+            }
+            return new Horde_Date($date->format('Y-m-d'));
 
         case self::TYPE_NINE:
-                if ($toEas) {
-                    return new Horde_Date($date->format('Y-m-d 00:00:00'), 'UTC');
-                } else {
-                    return $date;
-                }
+            if ($toEas) {
+                return new Horde_Date($date->format('Y-m-d 00:00:00'), 'UTC');
+            }
+            $date = new Horde_Date($date->format('Y-m-d'));
+            return $date->setTimezone('UTC');
 
         case self::TYPE_BLACKBERRY:
             if ($toEas) {
                 return new Horde_Date($date->format('Y-m-d 11:00:00'), 'UTC');
-            } else {
-                $date = new Horde_Date($date->format('Y-m-d'));
-                return $date->setTimezone('UTC');
             }
+            $date = new Horde_Date($date->format('Y-m-d'));
+            return $date->setTimezone('UTC');
 
         case self::TYPE_TOUCHDOWN:
         case self::TYPE_UNKNOWN:
@@ -676,6 +693,12 @@ class Horde_ActiveSync_Device
             case self::QUIRK_INCORRECTLY_SENDS_EMPTY_PICTURE_TAG:
                 return ($this->_isIos() && $this->getMajorVersion() >= 8 && $this->getMinorVersion() > 1);
 
+            case self::QUIRK_SUPPORTS_TNEF:
+                // These clients can support TNEF data naitively.
+                return (strpos($this->deviceType, 'MicrosoftOutlook') !== false ||
+                    strpos($this->deviceType, 'WindowsOutlook') != false ||
+                    strpos($this->userAgent, 'Outlook') !== false);
+
             default:
                 return false;
         }
@@ -700,6 +723,9 @@ class Horde_ActiveSync_Device
                 return self::TYPE_TOUCHDOWN;
             } else if ($this->_isNine()) {
                 return self::TYPE_NINE;
+            } else if (stripos($this->userAgent, 'Android-Mail') !== false) {
+                // Modern android OS uses GMAIL as the stock client.
+                return self::TYPE_GMAIL;
             } else if (stripos($this->userAgent, 'Android') !== false) {
                 return $this->deviceType;
             } else {
@@ -820,4 +846,8 @@ class Horde_ActiveSync_Device
         return false;
     }
 
+    public function __toString()
+    {
+        return serialize($this->_properties);
+    }
 }
