@@ -1,11 +1,8 @@
 <?php
 /**
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -15,11 +12,8 @@
  * of a Horde_ActiveSync_Message_Mail object.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -89,6 +83,10 @@ class Horde_ActiveSync_Imap_EasMessageBuilder
         $this->_airsyncBody = Horde_ActiveSync::messageFactory('AirSyncBaseBody');
         $this->_logger = Horde_ActiveSync::_wrapLogger($logger);
         $this->_procid = getmypid();
+
+        if ($this->_version > Horde_ActiveSync::VERSION_TWOFIVE) {
+            $this->_easMessage->airsyncbasenativebodytype = $this->_mbd->nativeBodyType;
+        }
     }
 
     /**
@@ -252,7 +250,12 @@ class Horde_ActiveSync_Imap_EasMessageBuilder
         reset($ids);
         $part1_id = next($ids);
         $part2_id = Horde_Mime::mimeIdArithmetic($part1_id, 'next');
-        $lines = explode(chr(13), $this->_imapMessage->getBodyPart($part2_id, array('decode' => true)));
+        try {
+            $lines = explode(chr(13), $this->_imapMessage->getBodyPart($part2_id, array('decode' => true)));
+        } catch (Horde_ActiveSync_Exception $e) {
+            return;
+        }
+
         switch ($part->getContentTypeParameter('report-type')) {
         case 'delivery-status':
             foreach ($lines as $line) {
@@ -304,10 +307,15 @@ class Horde_ActiveSync_Imap_EasMessageBuilder
 
         // Parse the iTip.
         $vCal = new Horde_Icalendar();
-        if ($vCal->parsevCalendar($data, 'VCALENDAR', $mime_part->getCharset())) {
-            $classes = $vCal->getComponentClasses();
-        } else {
-            $classes = array();
+        try {
+            if ($vCal->parsevCalendar($data, 'VCALENDAR', $mime_part->getCharset())) {
+                $classes = $vCal->getComponentClasses();
+            } else {
+                $classes = array();
+            }
+        } catch (Horde_Icalendar_Exception $e) {
+            $this->_logger->err($e->getMessage());
+            return;
         }
 
         // Exit if we can't parse/find any data.
@@ -507,7 +515,7 @@ class Horde_ActiveSync_Imap_EasMessageBuilder
 
         // First, see if we are EAS 2.5
         if ($options['protocolversion'] == Horde_ActiveSync::VERSION_TWOFIVE) {
-            return new Horde_ActiveSync_Imap_EasMessageBuilder_TwoFive($imap_message, $options);
+            return new Horde_ActiveSync_Imap_EasMessageBuilder_TwoFive($imap_message, $options, $logger);
         }
 
         switch ($mbd->getBodyTypePreference()) {
