@@ -1,11 +1,8 @@
 <?php
 /**
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -17,11 +14,8 @@
  * Some Mime parsing code taken from Imp_Contents.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -33,6 +27,8 @@
  */
 class Horde_ActiveSync_Imap_Message
 {
+    const OPTIONS_DECODE_TNEF = "decode_tnef";
+
     /**
      * Message data.
      *
@@ -75,6 +71,20 @@ class Horde_ActiveSync_Imap_Message
      */
     protected $_mbd;
 
+    /**
+     * Additional message options
+     *
+     * @var array @since 2.40.0
+     */
+    protected $_options;
+
+    /**
+     * Flag to indicate the mime map was processed for potential TNEF parts.
+     *
+     * @var boolean
+     */
+    protected $_tnefPrepared = false;
+
     /**
      * Constructor
      *
@@ -83,15 +93,26 @@ class Horde_ActiveSync_Imap_Message
      * @param Horde_Imap_Client_Data_Fetch $data  The data returned from a FETCH
      *                                            must contain at least uid,
      *                                            structure and flags.
+     * @param array $options                      Additional Options
+     *                                            @Since 2.40.0
+     *  ATTACHEMENT_OPTIONS_DECODE_TNEF  - if true will attempt to decode TNEF
+     *      MIME Parts. If false, will return TNEF data as-is. DEFAULT: True.
      */
     public function __construct(
         Horde_Imap_Client_Base $imap,
         Horde_Imap_Client_Mailbox $mbox,
-        Horde_Imap_Client_Data_Fetch $data)
+        Horde_Imap_Client_Data_Fetch $data,
+        array $options = array())
     {
         $this->_imap = $imap;
         $this->_mbox = $mbox;
         $this->_data = $data;
+        $this->_options = array_merge(
+            array(self::OPTIONS_DECODE_TNEF => true),
+            $options
+        );
+
+        $this->_prepareTnef();
     }
 
     public function __destruct()
@@ -308,9 +329,7 @@ class Horde_ActiveSync_Imap_Message
             $type = $part->getType();
             $id = $part->getMimeId();
             if ($this->isAttachment($id, $type)) {
-                if ($type != 'application/ms-tnef' || (!$mime_part = $this->_decodeTnefData($id))) {
-                    $mime_part = $this->getMimePart($id, array('nocontents' => true));
-                }
+                $mime_part = $this->getMimePart($id, array('nocontents' => true));
                 $ret[] = $this->_buildEasAttachmentFromMime($id, $mime_part, $version);
                 $mime_part = null;
             }
@@ -342,10 +361,14 @@ class Horde_ActiveSync_Imap_Message
         $atc->attsize = intval($mime_part->getBytes(true));
         $atc->attname = $this->_mbox . ':' . $this->uid . ':' . $id;
         $atc->displayname = $this->getPartName($mime_part, true);
-        $atc->attmethod = in_array($mime_part->getType(), array('message/rfc822', 'message/disposition-notification'))
+        $atc->attmethod = in_array($mime_part->getType(), array('message/disposition-notification'))
             ? Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_EMBEDDED
             : Horde_ActiveSync_Message_AirSyncBaseAttachment::ATT_TYPE_NORMAL;
 
+        if ($mime_part->getType() == 'message/rfc822') {
+            $atc->displayname .= ".eml";
+        }
+
         return $atc;
     }
 
@@ -356,8 +379,9 @@ class Horde_ActiveSync_Imap_Message
      *                                        Horde_Mime_Part object containing
      *                                        the TNEF attachment.
      *
-     * @return Horde_Mime_Part  The multipart/mixed MIME part containing any
-     *                          attachment data we can decode.
+     * @return Horde_Mime_Part|boolean  The multipart/mixed MIME part containing
+     *                                  any attachment data we can decode or
+     *                                  false on failure.
      */
     protected function _decodeTnefData($data)
     {
@@ -369,6 +393,9 @@ class Horde_ActiveSync_Imap_Message
         } else {
             $mime_part = $data;
         }
+        $wrapper->setName($mime_part->getName());
+        $wrapper->setMimeId($mime_part->getMimeId());
+
         $tnef_parser = Horde_Compress::factory('Tnef');
         try {
             $tnef_data = $tnef_parser->decompress($mime_part->getContents());
@@ -393,6 +420,8 @@ class Horde_ActiveSync_Imap_Message
             $wrapper->addPart($tmp_part);
         }
 
+        $wrapper->buildMimeIds();
+
         return $wrapper;
     }
 
@@ -411,19 +440,39 @@ class Horde_ActiveSync_Imap_Message
                 if ($mpart->getType() == 'text/calendar') {
                     $mpart->setDisposition('inline');
                 }
-                if ($mpart->getType() != 'application/ms-tnef' ||
-                    ($mpart->getType() == 'application/ms-tnef' && !$part = $this->_decodeTnefData($mpart))) {
-                    $part = $mpart;
-                }
-                $mime_parts[] = $part;
+                $mime_parts[] = $mpart;
                 $mpart = null;
-                $part = null;
             }
         }
 
         return $mime_parts;
     }
 
+    /**
+     * Check the mime structure for any TNEF data, and if neccessary attempt
+     * to decode data and inject back into the base mime part.
+     */
+    protected function _prepareTnef()
+    {
+        if ($this->_tnefPrepared) {
+            return;
+        }
+        $map = $this->basePart->contentTypeMap();
+        foreach ($map as $id => $type) {
+            if ($type == 'application/ms-tnef' &&
+                !empty($this->_options[self::OPTIONS_DECODE_TNEF])) {
+
+                $mpart = $this->getMimePart($id);
+                $tnef_part = $this->_decodeTnefData($mpart);
+                if ($tnef_part) {
+                    $this->basePart->alterPart($id, $tnef_part);
+                }
+            }
+        }
+        $this->basePart->buildMimeIds(null, false);
+        $this->_tnefPrepared = true;
+    }
+
     /**
      * Fetch a part of a MIME message.
      *
@@ -451,14 +500,17 @@ class Horde_ActiveSync_Imap_Message
             empty($options['nocontents']) &&
             !$part->getContents(array('stream' => true))) {
 
-            $body = $this->getBodyPart(
-                $id,
-                array(
-                    'decode' => true,
-                    'length' => empty($options['length']) ? null : $options['length'],
-                    'stream' => true)
-            );
-            $part->setContents($body, array('encoding' => $this->_lastBodyPartDecode, 'usestream' => true));
+            try {
+                $body = $this->getBodyPart(
+                    $id,
+                    array(
+                        'decode' => true,
+                        'length' => empty($options['length']) ? null : $options['length'],
+                        'stream' => true)
+                );
+                $part->setContents($body, array('encoding' => $this->_lastBodyPartDecode, 'usestream' => true));
+            } catch (Horde_ActiveSync_Exception $e) {
+            }
         }
 
         return $part;
@@ -543,6 +595,7 @@ class Horde_ActiveSync_Imap_Message
      *
      * @return mixed  The text of the part or a stream resource if 'stream'
      *                is true.
+     * @throws  Horde_ActiveSync_Exception when a bodypart cannot be found.
      * @todo Simplify by removing 'mimeheaders' parameter (not used).
      */
     public function getBodyPart($id, $options)
@@ -581,6 +634,10 @@ class Horde_ActiveSync_Imap_Message
             array('ids' => new Horde_Imap_Client_Ids(array($this->uid)))
         );
 
+        if (empty($fetch_res[$this->uid])) {
+            throw new Horde_ActiveSync_Exception('Message not found!');
+        }
+
         if (empty($options['mimeheaders'])) {
             $this->_lastBodyPartDecode = $fetch_res[$this->uid]->getBodyPartDecode($id);
             return $fetch_res[$this->uid]->getBodyPart($id, $options['stream']);
