@@ -3,11 +3,8 @@
  * Horde_ActiveSync_Message_ResolveRecipientsPicture::
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2013-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2013-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -16,11 +13,8 @@
  * data to send in a RESOLVERECIPIENTS response.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2013-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2013-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
@@ -36,8 +30,8 @@ class Horde_ActiveSync_Message_ResolveRecipientsPicture extends Horde_ActiveSync
      * @var array
      */
     protected $_mapping = array(
-        Horde_ActiveSync_Message_ResolveRecipients::TAG_STATUS => array(self::KEY_ATTRIBUTE => 'status'),
-        Horde_ActiveSync_Message_ResolveRecipients::TAG_DATA   => array(self::KEY_ATTRIBUTE => 'data')
+        Horde_ActiveSync_Request_ResolveRecipients::TAG_STATUS => array(self::KEY_ATTRIBUTE => 'status'),
+        Horde_ActiveSync_Request_ResolveRecipients::TAG_DATA   => array(self::KEY_ATTRIBUTE => 'data', self::KEY_TYPE => 'opaque')
     );
 
     /**
@@ -50,4 +44,4 @@ class Horde_ActiveSync_Message_ResolveRecipientsPicture extends Horde_ActiveSync
         'data'   => false
     );
 
-}
\ No newline at end of file
+}
