@@ -3,11 +3,8 @@
  * Horde_ActiveSync_Request_Base::
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2010-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2010-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -15,11 +12,8 @@
  * Base class for handlig ActiveSync requests.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2010-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2010-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal
@@ -235,9 +229,10 @@ abstract class Horde_ActiveSync_Request_Base
     public function handle()
     {
         $this->_logger->info(sprintf(
-            '%sRequest being handled for device: %s, Supporting protocol version: %s, Using Horde_ActiveSync v%s',
+            '%sRequest being handled for device: %s, of type: %s, supporting protocol version: %s, using Horde_ActiveSync v%s',
             str_repeat('-', 10),
             $this->_device->id,
+            $this->_device->deviceType,
             $this->_device->version,
             Horde_ActiveSync::LIBRARY_VERSION)
         );
@@ -245,6 +240,12 @@ abstract class Horde_ActiveSync_Request_Base
             'GET VARIABLES: %s',
             print_r($this->_activeSync->getGetVars(), true))
         );
+        $this->_logger->meta(sprintf(
+            '%sDevice data: %s',
+            str_repeat('-', 10),
+            $this->_device->__toString()
+        ));
+
         try {
             return $this->_handle();
         } catch (Exception $e) {
