@@ -14,11 +14,8 @@
  *   Consult COPYING file for details
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -26,11 +23,8 @@
  * Handle Search requests.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2012-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2012-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  * @internal
@@ -87,6 +81,7 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
     const STORE_STATUS_NOTFOUND      = 6;
     const STORE_STATUS_CONNECTIONERR = 7;
     const STORE_STATUS_COMPLEX       = 8;
+    const STORE_STATUS_FOLDERSYNC    = 11;
 
     /**
      * @var Horde_ActiveSync_Collections
@@ -110,31 +105,35 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
             !$this->_decoder->getElementStartTag(self::SEARCH_STORE) ||
             !$this->_decoder->getElementStartTag(self::SEARCH_NAME)) {
 
-            $search_status = self::SEARCH_STATUS_ERROR;
+            throw new Horde_ActiveSync_Exception_InvalidRequest('Missing required SEARCH|STORE|NAME');
         }
+
         $search_name = $this->_decoder->getElementContent();
         if (!$this->_decoder->getElementEndTag()) {
-            $search_status = self::SEARCH_STATUS_ERROR;
-            $store_status = self::STORE_STATUS_PROTERR;
+            return false;
         }
 
         if (!$this->_decoder->getElementStartTag(self::SEARCH_QUERY)) {
-            $search_status = self::SEARCH_STATUS_ERROR;
-            $store_status = self::STORE_STATUS_PROTERR;
+            throw new Horde_ActiveSync_Exception_InvalidRequest('Missing required SEARCH_QUERY.');
         }
+
         $search_query = array();
         switch (Horde_String::lower($search_name)) {
         case 'documentlibrary':
         case 'mailbox':
-            $search_query['query'] = $this->_parseQuery();
+            if (!($search_query['query'] = $this->_parseQuery())) {
+                $search_status = self::SEARCH_STATUS_ERROR;
+                $store_status = self::STORE_STATUS_FOLDERSYNC;
+            }
             break;
         case 'gal':
             $search_query['query'] = $this->_decoder->getElementContent();
         }
+
         if (!$this->_decoder->getElementEndTag()) {
-            $search_status = self::SEARCH_STATUS_ERROR;
-            $store_status = self::STORE_STATUS_PROTERR;
+            return false;
         }
+
         $mime = Horde_ActiveSync::MIME_SUPPORT_NONE;
         if ($this->_decoder->getElementStartTag(self::SEARCH_OPTIONS)) {
             $searchbodypreference = array();
@@ -142,8 +141,7 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
                 if ($this->_decoder->getElementStartTag(self::SEARCH_RANGE)) {
                     $search_query['range'] = $this->_decoder->getElementContent();
                     if (!$this->_decoder->getElementEndTag()) {
-                        $search_status = self::SEARCH_STATUS_ERROR;
-                        $store_status = self::STORE_STATUS_PROTERR;
+                        return false;
                     }
                 }
                 if ($this->_decoder->getElementStartTag(self::SEARCH_DEEPTRAVERSAL)) {
@@ -227,12 +225,11 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
         }
 
         if (!$this->_decoder->getElementEndTag()) { //store
-            $search_status = self::SEARCH_STATUS_ERROR;
-            $store_status = self::STORE_STATUS_PROTERR;
+            return false;
         }
+
         if (!$this->_decoder->getElementEndTag()) { //search
-            $search_status = self::SEARCH_STATUS_ERROR;
-            $store_status = self::STORE_STATUS_PROTERR;
+            return false;
         }
 
         $search_query['range'] = empty($search_query['range']) ? '0-99' : $search_query['range'];
@@ -244,13 +241,16 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
         }
 
         // Get search results from backend
-        $search_result = $this->_driver->getSearchResults($search_name, $search_query);
-
-        // @TODO: Remove for H6. Total should be returned from the search call,
-        // if it's not, do the best we can an use the count of results from
-        // this page.
-        if (empty($search_result['total'])) {
-            $search_result['total'] = count($search_result['rows']);
+        if ($search_query['query']) {
+            $search_result = $this->_driver->getSearchResults($search_name, $search_query);
+            // @TODO: Remove for H6. Total should be returned from the search call,
+            // if it's not, do the best we can an use the count of results from
+            // this page.
+            if (empty($search_result['total'])) {
+                $search_result['total'] = count($search_result['rows']);
+            }
+        } else {
+            $search_result = array();
         }
 
         /* Send output */
@@ -293,7 +293,7 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
                     $this->_encoder->endTag();
 
                     $this->_encoder->endTag();
-                    continue;
+                    break;
 
                 case 'gal':
                     $this->_encoder->startTag(self::SEARCH_RESULT);
@@ -408,7 +408,7 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
      *
      * @param boolean $subquery  Parsing a subquery.
      *
-     * @return array
+     * @return array | false on error
      */
     protected function _parseQuery($subquery = null)
     {
@@ -446,7 +446,11 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
             default:
                 if (($query[$type] = $this->_decoder->getElementContent())) {
                     if ($type == Horde_ActiveSync::SYNC_FOLDERID) {
-                        $query['serverid'] = $this->_collections->getBackendIdForFolderUid($query[$type]);
+                        try {
+                            $query['serverid'] = $this->_collections->getBackendIdForFolderUid($query[$type]);
+                        } catch (Horde_ActiveSync_Exception_FolderGone $e) {
+                            $this->_logger->err($e->getMessage());
+                        }
                     }
                     $this->_decoder->getElementEndTag();
                 } else {
@@ -470,7 +474,7 @@ class Horde_ActiveSync_Request_Search extends Horde_ActiveSync_Request_SyncBase
     {
         $this->_decoder->getElementEndTag(); // end SYNC_ITEMOPERATIONS_ITEMOPERATIONS
         $this->_encoder->startWBXML($this->_activeSync->multipart);
-        $this->_encoder->startTag(self::ITEMOPERATIONS_ITEMOPERATIONS);
+        $this->_encoder->startTag(self::SEARCH_SEARCH);
         $this->_encoder->startTag(self::SEARCH_STATUS);
         $this->_encoder->content($this->_statusCode);
         $this->_encoder->endTag();
