@@ -3,11 +3,8 @@
  * Horde_ActiveSync_State_Base::
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2009-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2009-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -15,11 +12,8 @@
  * Base class for managing everything related to device state
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2009-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2009-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -514,6 +508,12 @@ abstract class Horde_ActiveSync_State_Base
                 default:
                     $client_timestamps = $this->_getPIMChangeTS($changes);
                     $cnt = count($changes);
+
+                    if (!empty($changes) && !is_array($changes[0])) {
+                        $this->_changes = $changes;
+                        break;
+                    }
+
                     for ($i = 0; $i < $cnt; $i++) {
                         if (empty($client_timestamps[$changes[$i]['id']])) {
                             $this->_changes[] = $changes[$i];
