@@ -14,11 +14,8 @@
  *   Consult COPYING file for details
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2009-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2009-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -27,11 +24,8 @@
  * server to client.
  *
  * @license   http://www.horde.org/licenses/gpl GPLv2
- *            NOTE: According to sec. 8 of the GENERAL PUBLIC LICENSE (GPL),
- *            Version 2, the distribution of the Horde_ActiveSync module in or
- *            to the United States of America is excluded from the scope of this
- *            license.
- * @copyright 2009-2017 Horde LLC (http://www.horde.org)
+ *
+ * @copyright 2009-2020 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  */
@@ -205,18 +199,23 @@ class Horde_ActiveSync_Wbxml_Encoder extends Horde_ActiveSync_Wbxml
      * @param mixed $content  The value to output for this tag. A string or
      *                        a stream resource.
      */
-    public function content($content)
+    public function content($content, $opaque = false)
     {
-        // Don't try to send a string containing \0 - it's the wbxml string
-        // terminator.
-        if (!is_resource($content)) {
-            $content = str_replace("\0", '', $content);
-            if ('x' . $content == 'x') {
-                return;
+        if (!$opaque) {
+            // Don't try to send a string containing \0 - it's the wbxml string
+            // terminator.
+            if (!is_resource($content)) {
+                $content = str_replace("\0", '', $content);
+                if ('x' . $content == 'x') {
+                    return;
+                }
             }
         }
+
         $this->_outputStack();
-        $this->_content($content);
+
+
+        $this->_content($content, $opaque);
 
         if (is_resource($content)) {
             fclose($content);
@@ -299,7 +298,7 @@ class Horde_ActiveSync_Wbxml_Encoder extends Horde_ActiveSync_Wbxml
      *
      * @param mixed $content  A string or stream resource to write to the output
      */
-    private function _content($content)
+    private function _content($content, $opaque = false)
     {
         if (!is_resource($content)) {
             if ($this->_logLevel == self::LOG_PROTOCOL &&
@@ -317,6 +316,18 @@ class Horde_ActiveSync_Wbxml_Encoder extends Horde_ActiveSync_Wbxml
                 $this->_logContent('[STREAM]');
             }
         }
+        if ($opaque) {
+            if (!is_resource($content)) {
+                $len = strlen($content);
+            } else {
+                $len = 0;
+            }
+            $this->_outByte(self::OPAQUE);
+            $this->_outMBUInt($len);
+            //$content = base64_encode($content);
+            $this->_stream->add($content);
+            return;
+        }
         $this->_outByte(self::STR_I);
         $this->_outTermStr($content);
     }
